@@ -2,8 +2,8 @@
 
 import os
 
+import deprecation
 import google.protobuf.wrappers_pb2 as wrappers
-
 from cognite.seismic._api.api import API
 
 if not os.getenv("READ_THE_DOCS"):
@@ -14,8 +14,8 @@ if not os.getenv("READ_THE_DOCS"):
 
 
 class SliceAPI(API):
-    def __init__(self, query, metadata):
-        super().__init__(query=query, metadata=metadata)
+    def __init__(self, query):
+        super().__init__(query=query)
 
     @staticmethod
     def _build_range(r_from=None, r_to=None):
@@ -31,10 +31,10 @@ class SliceAPI(API):
         """Returns a slice from the gRPC service
 
         Args:
-            line_select: A data type specifying inline or crossline
+            line_select: A data type specifying inline or xline
             file_identifier: A data type identifying the file
             include_headers (bool): Include headers in response
-            ort_range: The range for which to get the slice (specified as line_select is inline if crossline and vice versa)
+            ort_range: The range for which to get the slice (specified as line_select is inline if xline and vice versa)
 
         Returns:
             Traces
@@ -42,8 +42,16 @@ class SliceAPI(API):
         request = LineSliceQueryRequest(
             file=file_identifier, line=line_select, include_trace_header=include_headers, range=ort_range
         )
-        return Trace2DList((i for i in self.query.GetSliceByLine(request, metadata=self.metadata)))
 
+        def get_stream():
+            return self.query.GetSliceByLine(request)
+
+        return Trace2DList(get_stream)
+
+    @deprecation.deprecated(
+        deprecated_in="0.1.57",
+        details='Use volume API instead, for example: client.volume.get(file_id="id", inline_range=(inline, inline))',
+    )
     def get_inline(self, inline, file_id=None, file_name=None, include_headers=False, from_line=None, to_line=None):
         """Gets the traces of an inline in a given file.
 
@@ -52,8 +60,8 @@ class SliceAPI(API):
             file_id (str, optional): File can be specified either by name or id (id will be used first if both are provided)
             file_name (str, optional): File can be specified either by name or id (id will be used first if both are provided)
             include_headers (bool, optional): Whether or not to include the trace headers in the response
-            from_line (int, optional): Include only crosslines equal or greater to this in the slice
-            to_line (int, optional): Include only crosslines equal or less than this in the slice
+            from_line (int, optional): Include only xlines equal or greater to this in the slice
+            to_line (int, optional): Include only xlines equal or less than this in the slice
 
         Returns:
             Series of traces where each contains its inline, xline and the values.
@@ -66,13 +74,15 @@ class SliceAPI(API):
             ort_range=self._build_range(from_line, to_line),
         )
 
-    def get_crossline(
-        self, crossline, file_id=None, file_name=None, include_headers=False, from_line=None, to_line=None
-    ):
-        """Gets the traces of a crossline in a given file.
+    @deprecation.deprecated(
+        deprecated_in="0.1.57",
+        details='Use volume API instead, for example: client.volume.get(file_id="id", xline_range=(xline, xline))',
+    )
+    def get_xline(self, xline, file_id=None, file_name=None, include_headers=False, from_line=None, to_line=None):
+        """Gets the traces of a xline in a given file.
 
         Args:
-            crossline (int): Crossline number
+            xline (int): xline number
             file_id (str, optional): File can be specified either by name or id (id will be used first if both are provided)
             file_name (str, optional): File can be specified either by name or id (id will be used first if both are provided)
             include_headers (bool, optional): Whether or not to include the trace headers in the response
@@ -83,7 +93,7 @@ class SliceAPI(API):
             The line can be converted to a 2D numpy array with just the values calling .to_array() on it
         """
         return self._get_slice(
-            line_select=LineSelect(xline=crossline),
+            line_select=LineSelect(xline=xline),
             file_identifier=self.identify(file_id, file_name),
             include_headers=include_headers,
             ort_range=self._build_range(from_line, to_line),
@@ -94,19 +104,19 @@ class SliceAPI(API):
     ):
         """Returns a series of traces on an arbitrary line starting from (x0, yo) and ending in (x1, y1)
 
-        Currently, all the traces are snapped to the grid inlines or crosslines
+        Currently, all the traces are snapped to the grid inlines or xlines
 
         Args:
             file_id (str, optional): File can be specified either by name or id (id will be used first if both are provided)
             file_name (str, optional): File can be specified either by name or id (id will be used first if both are provided)
             crs (str): Specify the CRS in which the coordinates x0, y0, x1 and y1 are given (Ex.: "EPSG:23031")
             interpolation_method (int, optional): Specify the interpolation_method. Currently possible are 0 for snapping the traces to the nearest
-                    grid points (inlines or crosslines) or 1 for weighing the nearest traces on each side of the line
+                    grid points (inlines or xlines) or 1 for weighing the nearest traces on each side of the line
         Returns:
             Series of traces composing the line.
             The line can be converted to a 2D numpy array with just the values calling .to_array() on it
         """
-        if not x0 or not y0 or not x1 or not y1 or not crs:
+        if x0 is None or y0 is None or x1 is None or y1 is None or not crs:
             raise Exception("x0, y0, x1, y1 and crs must be specified")
         p0 = f"{x0} {y0}"
         p1 = f"{x1} {y1}"
@@ -114,4 +124,8 @@ class SliceAPI(API):
         line = Geometry(crs=CRS(crs=crs), wkt=Wkt(geometry=linestring))
         file = self.identify(file_id, file_name)
         request = GeometrySliceQueryRequest(file=file, arbitrary_line=line, interpolation_method=interpolation_method)
-        return Trace2DList((i for i in self.query.GetSliceByGeometry(request, metadata=self.metadata)))
+
+        def get_stream():
+            return self.query.GetSliceByGeometry(request)
+
+        return Trace2DList(get_stream)
