@@ -14,8 +14,8 @@ if not os.getenv("READ_THE_DOCS"):
 
 
 class TimeSliceAPI(API):
-    def __init__(self, query, metadata):
-        super().__init__(query=query, metadata=metadata)
+    def __init__(self, query):
+        super().__init__(query=query)
 
     @staticmethod
     def _verify_input(crs: str = None, wkt: str = None, geo_json: str = None):
@@ -84,6 +84,6 @@ class TimeSliceAPI(API):
         req = GeometryTimeSliceQueryRequest(file=file, geometry=geo, z=wrappers.Int32Value(value=z))
 
         def get_stream():
-            return self.query.GetTimeSliceByGeometry(req, metadata=self.metadata)
+            return self.query.GetTimeSliceByGeometry(req)
 
         return SurfacePointList(get_stream)
