@@ -5,20 +5,18 @@ from enum import Enum
 from typing import Iterable, List, Mapping, NamedTuple, Optional, Tuple, Union
 
 import numpy as np
-from cognite.seismic._api.utility import Direction, LineRange, MaybeString
+from cognite.seismic._api.utility import Direction, LineRange, MaybeString, _egcd
 
 if not os.getenv("READ_THE_DOCS"):
     from cognite.seismic.protos.types_pb2 import CRS as CRSProto
+    from cognite.seismic.protos.types_pb2 import DeduceFromTraces as DeduceFromTracesProto
+    from cognite.seismic.protos.types_pb2 import DoubleTraceCoordinates as DoubleTraceCoordinatesProto
     from cognite.seismic.protos.types_pb2 import GeoJson
     from cognite.seismic.protos.types_pb2 import Geometry as GeometryProto
-    from cognite.seismic.protos.types_pb2 import IngestionSource, Wkt
-    from cognite.seismic.protos.types_pb2 import (
-        P6Transformation as P6TransformationProto,
-        DoubleTraceCoordinates as DoubleTraceCoordinatesProto,
-        DeduceFromTraces as DeduceFromTracesProto,
-        SurveyGridTransformation as SurveyGridTransformationProto,
-        TraceCorners,
-    )
+    from cognite.seismic.protos.types_pb2 import IngestionSource
+    from cognite.seismic.protos.types_pb2 import P6Transformation as P6TransformationProto
+    from cognite.seismic.protos.types_pb2 import SurveyGridTransformation as SurveyGridTransformationProto
+    from cognite.seismic.protos.types_pb2 import TraceCorners, Wkt
     from cognite.seismic.protos.v1.seismic_service_datatypes_pb2 import BinaryHeader as BinaryHeaderProto
     from cognite.seismic.protos.v1.seismic_service_datatypes_pb2 import TextHeader as TextHeaderProto
     from google.protobuf.json_format import MessageToDict
@@ -123,14 +121,20 @@ class RangeInclusive:
         self.step = step
 
     @classmethod
-    def from_linerange(cls, linerange: LineRange) -> "RangeInclusive":
+    def from_linerange(cls, linerange: Optional[LineRange]) -> Optional["RangeInclusive"]:
         """Construct a RangeInclusive from a (start, stop) or (start, stop, step) tuple"""
+        if linerange is None:
+            return None
         if len(linerange) == 2:
-            return cls(linerange[0], linerange[1], 1)
+            if linerange[0] < linerange[1]:
+                step = 1
+            else:
+                step = -1
+            return cls(linerange[0], linerange[1], step)
         elif len(linerange) == 3:
             return cls(linerange[0], linerange[1], linerange[2])
         else:
-            raise ValueError("Linerange must have 2 or 3 elements")
+            raise ValueError("LineRange must have 2 or 3 elements")
 
     @classmethod
     def widest(cls, ranges: Iterable[Tuple[int, int]]) -> "RangeInclusive":
@@ -148,11 +152,20 @@ class RangeInclusive:
 
     def index(self, line: int) -> int:
         """Compute the index of a given linenumber in this range"""
-        if line < self.start or line > self.stop:
-            raise ValueError(f"line {line} out of range")
-        if (line - self.start) % self.step != 0:
-            raise ValueError(f"line {line} incompatible with step {self.step} starting from {self.start}")
-        return (line - self.start) // self.step
+        out_of_range = ValueError(f"line {line} out of range")
+        incompatible = ValueError(f"line {line} incompatible with step {self.step} starting from {self.start}")
+        if self.step > 0:
+            if line < self.start or line > self.stop:
+                raise out_of_range
+            if (line - self.start) % self.step != 0:
+                raise incompatible
+            return (line - self.start) // self.step
+        else:
+            if line > self.start or line < self.stop:
+                raise out_of_range
+            if (line - self.start) % self.step != 0:
+                raise incompatible
+            return (self.start - line) // (-self.step)
 
     def to_linerange(self) -> LineRange:
         """Return a (start, stop) or a (start, stop, step) tuple"""
@@ -161,13 +174,39 @@ class RangeInclusive:
         else:
             return (self.start, self.stop, self.step)
 
+    def to_positive(self) -> "RangeInclusive":
+        """Return an equivalent RangeInclusive where the step size is always positive"""
+        if self.step > 0:
+            return self
+        else:
+            return RangeInclusive(self.stop, self.start, -self.step)
+
+    def intersection(self, other: "RangeInclusive") -> Optional["RangeInclusive"]:
+        """Compute the intersection of this range with another"""
+        this = self.to_positive()
+        other = other.to_positive()
+        (g, u, v) = _egcd(this.step, other.step)
+        if (this.start - other.start) % g != 0:
+            return None
+        step = (this.step * other.step) // g
+        sol = (this.start * v * other.step + other.start * u * this.step) // g
+        max_start = max(this.start, other.start)
+        # steps_to_start = ceil((max_start - sol) / step)
+        steps_to_start = (max_start - sol + step - 1) // step
+        start = sol + step * steps_to_start
+        stop = min(this.stop, other.stop)
+        if stop >= start:
+            return RangeInclusive(start, stop, step)
+        else:
+            return None
+
     def __len__(self) -> int:
         """Return the number of lines described by this range"""
         return (self.stop - self.start) // self.step + 1
 
     def __iter__(self):
         cur = self.start
-        while cur <= self.stop:
+        while (self.step > 0 and cur <= self.stop) or (self.step < 0 and cur >= self.stop):
             yield cur
             cur += self.step
 
@@ -178,7 +217,12 @@ class RangeInclusive:
         return f"RangeInclusive({self.start}, {self.stop}, {self.step})"
 
     def __eq__(self, other):
-        return self.start == other.start and self.stop == other.stop and self.step == other.step
+        same_start_stop = self.start == other.start and self.stop == other.stop
+        if self.start == self.stop:
+            # Singleton ranges, so it doesn't matter what the steps are
+            return same_start_stop
+        else:
+            return same_start_stop and self.step == other.step
 
 
 class VolumeDef:
@@ -245,33 +289,27 @@ class VolumeDef:
                 yield value
 
     @staticmethod
-    def _get_method_size(method, minor_range: Optional[Tuple[int, int]] = None) -> int:
+    def _get_method_size(method, minor_range: Optional[LineRange] = None) -> int:
         """Given a volumedef method, calculate the number of traces within it and minor_range"""
+        minor_range = RangeInclusive.from_linerange(minor_range)
         c = 0
         if "list" in method:
             if minor_range is None:
                 c += len(method["list"])
             else:
                 for minor in method["list"]:
-                    if minor < minor_range[0] or minor > minor_range[1]:
-                        continue
-                    c += 1
+                    if minor in minor_range:
+                        c += 1
         elif "range" in method:
-            start, stop = method["range"][0:2]
-            step = VolumeDef._get_range_step(method)
-            # Let's simplify to a positive range, shall we?
-            if start > stop:
-                start, stop = stop, start
-                step = -step
-            if minor_range is not None:
-                if start < minor_range[0]:
-                    # This computes the ceiling of (minor_range[0] - start) / step
-                    nsteps = (minor_range[0] - start + step - 1) // step
-                    start = start + step * nsteps
-                if stop > minor_range[1]:
-                    stop = minor_range[1]
-            if stop >= start:
-                c += (stop - start) // step + 1
+            v_range = RangeInclusive.from_linerange(method["range"])
+            if minor_range is None:
+                if v_range is not None:
+                    c += len(v_range)
+            else:
+                if v_range is not None:
+                    intersection = minor_range.intersection(v_range)
+                    if intersection is not None:
+                        c += len(intersection)
         else:
             raise Exception(f"The specified method {method} was not recognized.")
 
@@ -285,18 +323,14 @@ class VolumeDef:
             maximum = max(method["list"])
             return (minimum, maximum)
         elif "range" in method:
-            lower, upper = method["range"][0:2]
-            step = VolumeDef._get_range_step(method)
-
-            # Find highest multiple of step not greater than upper
-            end = lower + ((upper - lower) // step) * step
-
-            return tuple(sorted((lower, end)))  # Sort to handle negative-step volumedefs
+            range_ = RangeInclusive.from_linerange(method["range"]).to_positive()
+            # RangeInclusive will always have stop reachable from start
+            return (range_.start, range_.stop)
         else:
             raise Exception(f"The specified method {method} was not recognized.")
 
     def count_line_traces(
-        self, inline_range: Optional[Tuple[int, int]] = None, xline_range: Optional[Tuple[int, int]] = None
+        self, inline_range: Optional[LineRange] = None, xline_range: Optional[LineRange] = None
     ) -> Mapping[int, int]:
         """Count the number of traces by line, optionally restricted by ranges.
 
@@ -307,14 +341,17 @@ class VolumeDef:
             raise Exception(
                 f'The specified volumedef version {self.parsed["version"]} is not supported by this version of the Cognite SDK.'
             )
+
         output = {}
         if self.direction == Direction.INLINE:
             major_range, minor_range = inline_range, xline_range
         else:
             major_range, minor_range = xline_range, inline_range
+
+        major_range = RangeInclusive.from_linerange(major_range)
         for key, methods in self.parsed["lines"].items():
             key = int(key)
-            if major_range and (key < major_range[0] or key > major_range[1]):
+            if major_range and key not in major_range:
                 continue
             c = 0
             for method in methods:
@@ -323,14 +360,11 @@ class VolumeDef:
         return output
 
     def count_total_traces(
-        self, inline_range: Optional[Tuple[int, int]] = None, xline_range: Optional[Tuple[int, int]] = None
+        self, inline_range: Optional[LineRange] = None, xline_range: Optional[LineRange] = None
     ) -> int:
         """Count the total number of traces in this volumedef, optionally restricted by ranges."""
-        c = 0
         traces = self.count_line_traces(inline_range, xline_range)
-        for val in traces.values():
-            c += val
-        return c
+        return sum(traces.values())
 
     def get_major_range(self) -> Mapping[int, Tuple[int, int]]:
         """Return the min & max range of traces by major line, i.e. a mapping from major number to minor range.
@@ -488,9 +522,7 @@ class File:
 
     def __repr__(self):
         temporary = ", temporary" if self.is_temporary else ""
-        return (
-            f"File<id: {self.id}, name: {self.name}{temporary}, metadata: {metadata}, external_id: {self.external_id}>"
-        )
+        return f"File<id: {self.id}, name: {self.name}{temporary}, metadata: {self.metadata}, external_id: {self.external_id}>"
 
     @staticmethod
     def from_proto(proto) -> "File":
@@ -1154,6 +1186,7 @@ class SurveyCoverageSource(Enum):
 class Survey:
 
     id: str
+    id_int: int
     external_id: Optional[str]
     name: str
     seismic_ids: Optional[List[int]]
@@ -1178,8 +1211,10 @@ class Survey:
         grid_transformation,
         custom_coverage,
         survey_coverage_source,
+        id_int=-1,
     ):
         self.id = id
+        self.id_int = id_int
         self.external_id = external_id
         self.name = name
         self.seismic_ids = seismic_ids
@@ -1193,10 +1228,7 @@ class Survey:
 
     @staticmethod
     def from_proto(proto) -> "Survey":
-        survey_id = proto.survey.id
-        external_id = proto.survey.external_id.external_id if hasattr(proto.survey, "external_id") else None
-        name = proto.survey.name if hasattr(proto.survey, "name") else None
-        metadata = proto.survey.metadata if hasattr(proto.survey, "metadata") else None
+        survey = Survey.from_proto_survey(proto.survey)
         seismic_ids = proto.seismic_ids if hasattr(proto, "seismic_ids") else None
         seismic_store_ids = proto.seismic_store_ids if hasattr(proto, "seismic_store_ids") else None
         coverage = Geometry.from_proto(proto.coverage) if proto.HasField("coverage") else None
@@ -1215,19 +1247,29 @@ class Survey:
             proto.coverage_source if hasattr(proto, "coverage_source") else SurveyCoverageSource.UNSPECIFIED
         )
 
-        return Survey(
-            survey_id,
-            external_id,
-            name,
-            seismic_ids,
-            seismic_store_ids,
-            metadata,
-            coverage,
-            crs,
-            grid_transformation,
-            custom_coverage,
-            survey_coverage_source,
-        )
+        survey.seismic_ids = seismic_ids
+        survey.seismic_store_ids = seismic_store_ids
+        survey.coverage = coverage
+        survey.crs = crs
+        survey.grid_transformation = grid_transformation
+        survey.custom_coverage = custom_coverage
+        survey.survey_coverage_source = survey_coverage_source
+
+        return survey
+
+    @staticmethod
+    def from_proto_survey(proto) -> "Survey":
+        """From a Survey proto rather than the result of SearchSurvey"""
+        survey_id = proto.id
+        survey_id_int = proto.id_int
+        # Proto default value for ints is 0. Changing it to -1 to be a little clearer
+        if survey_id_int == 0:
+            survey_id_int = -1
+        external_id = proto.external_id.external_id if hasattr(proto, "external_id") else None
+        name = proto.name if hasattr(proto, "name") else None
+        metadata = proto.metadata if hasattr(proto, "metadata") else None
+
+        return Survey(survey_id, external_id, name, [], [], metadata, None, None, None, None, None)
 
     def __repr__(self):
         return "%s {\n%s\n}" % (type(self).__name__, "\n".join("%s = %s" % item for item in vars(self).items()))
